# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-03 10:31
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Questionnaire',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.TextField(blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Response',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.IntegerField(default=0)),
                ('question', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='likertblock.Question')),
            ],
        ),
        migrations.CreateModel(
            name='Submission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('submitted', models.DateTimeField(auto_now_add=True)),
                ('questionnaire', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='likertblock.Questionnaire')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='likert_submission', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='response',
            name='submission',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='likertblock.Submission'),
        ),
        migrations.AddField(
            model_name='question',
            name='questionnaire',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='likertblock.Questionnaire'),
        ),
        migrations.AlterOrderWithRespectTo(
            name='question',
            order_with_respect_to='questionnaire',
        ),
    ]



from django.test import TestCase
from django.contrib.auth.models import User
from likertblock.models import (
    Questionnaire, Question, Submission, Response)


class QuestionnaireTest(TestCase):
    def test_needs_submit(self):
        q = Questionnaire.objects.create()
        self.assertTrue(q.needs_submit())

    def test_clear_user_submissions(self):
        q = Questionnaire.objects.create()
        q.clear_user_submissions(None)


class QuestionTest(TestCase):
    def test_unicode(self):
        qn = Questionnaire.objects.create()
        q = Question.objects.create(questionnaire=qn, text="a question")
        self.assertEqual(str(q), "a question")


class SubmissionTest(TestCase):
    def test_unicode(self):
        qn = Questionnaire.objects.create()
        u = User.objects.create(username='foo')
        s = Submission.objects.create(user=u, questionnaire=qn)
        self.assertTrue("likert" in str(s))
        self.assertTrue("foo" in str(s))


class ResponseTest(TestCase):
    def test_likert_scale_label(self):
        qn = Questionnaire.objects.create()
        q = Question.objects.create(questionnaire=qn, text="a question")
        u = User.objects.create(username='foo')
        s = Submission.objects.create(user=u, questionnaire=qn)
        r = Response.objects.create(submission=s, question=q,
                                    value=0)
        self.assertEqual(r.likert_scale_label(), 'Strongly Disagree')
        r.value = 1
        r.save()
        self.assertEqual(r.likert_scale_label(), 'Somewhat Disagree')
        r.value = 2
        r.save()
        self.assertEqual(r.likert_scale_label(), 'Neutral')
        r.value = 3
        r.save()
        self.assertEqual(r.likert_scale_label(), 'Somewhat Agree')
        r.value = 4
        r.save()
        self.assertEqual(r.likert_scale_label(), 'Strongly Agree')



from .models import Questionnaire, Question
from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import render, get_object_or_404
from django.core.urlresolvers import reverse
from django.views.generic.base import View
from django.views.generic.detail import DetailView
from django.views.generic.edit import DeleteView


class EditQuestionnaireView(DetailView):
    model = Questionnaire


class DeleteQuestionView(DeleteView):
    model = Question

    def get_success_url(self):
        questionnaire = self.object.questionnaire
        return reverse("edit-questionnaire", args=[questionnaire.id])


class ReorderQuestionsView(View):
    def post(self, request, pk):
        questionnaire = get_object_or_404(Questionnaire, pk=pk)
        keys = request.GET.keys()
        question_keys = [int(k[len('question_'):]) for k in keys
                         if k.startswith('question_')]
        question_keys.sort()
        questions = [int(request.GET['question_' + str(k)])
                     for k in question_keys]
        questionnaire.update_questions_order(questions)
        return HttpResponse("ok")


class AddQuestionToQuestionnaireView(View):
    def post(self, request, pk):
        questionnaire = get_object_or_404(Questionnaire, pk=pk)
        form = questionnaire.add_question_form(request.POST)
        if form.is_valid():
            question = form.save(commit=False)
            question.questionnaire = questionnaire
            question.save()
        return HttpResponseRedirect(
            reverse("edit-questionnaire", args=[questionnaire.id]))


class EditQuestionView(View):
    template_name = 'likertblock/edit_question.html'

    def get_object(self, pk):
        return get_object_or_404(Question, pk=pk)

    def get(self, request, pk):
        return render(
            request,
            self.template_name,
            dict(question=self.get_object(pk)))

    def post(self, request, pk):
        question = self.get_object(pk)
        form = question.edit_form(request.POST)
        question = form.save(commit=False)
        question.save()
        return HttpResponseRedirect(reverse("likert-edit-question",
                                            args=[question.id]))



from django import forms
from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericRelation
from django.core.urlresolvers import reverse
from django.db import models
from pagetree.models import PageBlock


class Questionnaire(models.Model):
    pageblocks = GenericRelation(PageBlock)
    description = models.TextField(blank=True)

    display_name = "Likert"
    template_file = "likertblock/likertblock.html"
    exportable = True
    importable = True

    def pageblock(self):
        return self.pageblocks.all()[0]

    def __unicode__(self):
        return unicode(self.pageblock())

    def needs_submit(self):
        return True

    def submit(self, user, data):
        s = Submission.objects.create(questionnaire=self, user=user)
        for k in data.keys():
            if k.startswith('likert-question'):
                qid = int(k[len('likert-question'):])
                question = Question.objects.get(id=qid)
                Response.objects.create(
                    submission=s,
                    question=question,
                    value=data[k])

    def redirect_to_self_on_submit(self):
        return True

    def unlocked(self, user):
        return Submission.objects.filter(
            questionnaire=self, user=user).count() > 0

    def edit_form(self):
        class EditForm(forms.Form):
            description = forms.CharField(widget=forms.widgets.Textarea(),
                                          initial=self.description)
            alt_text = ("<a href=\"" + reverse("edit-questionnaire",
                                               args=[self.id])
                        + "\">manage questions</a>")
        return EditForm()

    def update_questions_order(self, question_ids):
        self.set_question_order(question_ids)

    def clear_user_submissions(self, user):
        Submission.objects.filter(user=user, questionnaire=self).delete()

    @classmethod
    def add_form(self):
        class AddForm(forms.Form):
            description = forms.CharField(widget=forms.widgets.Textarea())
        return AddForm()

    @classmethod
    def create(self, request):
        return Questionnaire.objects.create(
            description=request.POST.get('description', ''))

    @classmethod
    def create_from_dict(self, d):
        q = Questionnaire.objects.create(
            description=d.get('description', ''))
        q.import_from_dict(d)
        return q

    def edit(self, vals, files):
        self.description = vals.get('description', '')
        self.save()

    def add_question_form(self, request=None):
        return QuestionForm(request)

    def as_dict(self):
        return dict(
            description=self.description,
            questions=[q.as_dict() for q in self.question_set.all()]
        )

    def import_from_dict(self, d):
        self.description = d['description']
        self.save()
        self.submission_set.all().delete()
        self.question_set.all().delete()
        for q in d['questions']:
            Question.objects.create(
                questionnaire=self,
                text=q['text'])

    def summary_render(self):
        if len(self.description) < 61:
            return self.description
        else:
            return self.description[:61] + "..."


class Question(models.Model):
    questionnaire = models.ForeignKey(Questionnaire)
    text = models.TextField(blank=True)

    class Meta:
        order_with_respect_to = 'questionnaire'

    def __unicode__(self):
        return self.text

    def display_number(self):
        return self._order + 1

    def edit_form(self, request=None):
        return QuestionForm(request, instance=self)

    def user_responses(self, user):
        qs = Submission.objects.filter(
            user=user,
            questionnaire=self.questionnaire).order_by("-submitted")
        if len(qs) == 0:
            return Response.objects.none()
        else:
            submission = qs[0]
            return Response.objects.filter(
                question=self,
                submission=submission)

    def as_dict(self):
        return dict(text=self.text)


class Submission(models.Model):
    questionnaire = models.ForeignKey(Questionnaire)
    user = models.ForeignKey(User, related_name='likert_submission')
    submitted = models.DateTimeField(auto_now_add=True)

    def __unicode__(self):
        return "likert %d submission by %s at %s" % (
            self.questionnaire.id, unicode(self.user), self.submitted)


class Response(models.Model):
    question = models.ForeignKey(Question)
    submission = models.ForeignKey(Submission)
    value = models.IntegerField(default=0)

    def likert_scale_label(self):
        return [
            'Strongly Disagree',
            'Somewhat Disagree',
            'Neutral',
            'Somewhat Agree',
            'Strongly Agree',
        ][self.value]


class QuestionForm(forms.ModelForm):
    class Meta:
        model = Question
        exclude = ("questionnaire",)
        fields = ('text',)



from django.conf.urls import patterns
from .views import (
    EditQuestionnaireView, DeleteQuestionView, ReorderQuestionsView,
    AddQuestionToQuestionnaireView, EditQuestionView,
)

urlpatterns = patterns(
    'likertblock.views',
    (r'^edit_questionnaire/(?P<pk>\d+)/$', EditQuestionnaireView.as_view(),
     {}, 'edit-questionnaire'),
    (r'^edit_questionnaire/(?P<pk>\d+)/add_question/$',
     AddQuestionToQuestionnaireView.as_view(), {},
     'add-question-to-questionnaire'),
    (r'^edit_question/(?P<pk>\d+)/$', EditQuestionView.as_view(), {},
     'likert-edit-question'),
    (r'^delete_question/(?P<pk>\d+)/$', DeleteQuestionView.as_view(), {},
     'likert-delete-question'),
    (r'^reorder_questions/(?P<pk>\d+)/$', ReorderQuestionsView.as_view(), {},
     'likert-reorder-questions'),
)



from django import template
from likertblock.models import Response, Submission

register = template.Library()


class GetQuestionResponseNode(template.Node):
    def __init__(self, question, var_name):
        self.question = question
        self.var_name = var_name

    def render(self, context):
        q = context[self.question]
        u = context['request'].user
        questionnaire = q.questionnaire
        r = Submission.objects.filter(
            questionnaire=questionnaire, user=u).order_by("-submitted")
        if r.count() == 0:
            return ''
        submission = r[0]
        r = Response.objects.filter(question=q, submission=submission)
        if r.count() > 0:
            context[self.var_name] = r[0]
        else:
            context[self.var_name] = None
        return ''


@register.tag('getquestionresponse')
def getquestionresponse(parser, token):
    question = token.split_contents()[1:][0]
    var_name = token.split_contents()[1:][2]
    return GetQuestionResponseNode(question, var_name)
